# linksubmit.pl - Submit Link Job
#
# Authors:  Jane Huang
# Last updated:  7/14/07
#           Xiao Lin Zhang 8/17/09
#      Xiao Lin Zhang 28/6/10
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is spawned from the bccpmbld_main.pl script for each COBOL
# file that was selected to be linked.  
#
# The calling syntax for this script is:
# 	Usage: linksubmit.pl [-1] -d directory_path -f filename
# Where;
# 	-1 indicates that the linked-edit list is one file which causes
# 	   the log output to be displayed to stdout as well as the log file
# 	-d identifies the directory where the script and logs exist
# 	-f identifies the file to be compiled
#
#
# This script can be limited to specific users during development by using the
# following lines and changing the username strings.
# In this example an user id table 'userid' is used and queried from bccpbmod.pm.
# If using your site security system, this step may be skipped.
#
# my $user = $ENV{'USERNAME'};
# exit 0 if ((lc $user ne "USER1") && (lc $user ne "USER2"));
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# ****************************************************************************** 
use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Getopt::Std;
use bccpbmod;

my $debug      = $bccpbmod::debug;     # for verbose debugging output
my $debug      = 1;     # for verbose debugging output

my $rccVerbosity  = "";
$rccVerbosity  = '-V -V -V' if $debug;

my $sysout       = $bccpbmod::sysout;
$sysout = 'RCCEXT=RCCOUT,DISP=(NEW,CATLG,DELETE),' if $debug;
my $sysout1 = 'SPACE=(32000,(30,30))' if $debug;

my $jcl;	    # JCL filename
my $ncpycnt = 7;    # number of CICS NewCopy steps
my $lcnt    = 0;    # general loop counter

# #############################################################################
# process input arguments
#
#
  
my $usage = "linksubmit [-1] -d dir -f file";         

my $compile_1 = 0;  # Link-edit from main consisted of one file
my $dir_file  = ''; # directory where the file resides
my $filename  = ''; # filename - full or relative path

if (!getopts('1d:f:') || $opt_h) {
    print "$usage \n";
    exit(0);
}


$compile_1 = $opt_1;
$dir_file  = $opt_d;
$filename  = $opt_f;

my $current_dir = `cd`;
print "current dir =  $current_dir \n" if $debug;

print "Link script: One file = $compile_1 \n" if $debug;
print "Link script: filename = $filename \n" if $debug;
print "Link script: dir_file = $dir_file \n" if $debug;


# #############################################################################
# Retrieve the environment information from the CC Project attribute
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.  If you determine to use the optional variables or add new variables,*
# * you have to update all related scripts correspondently.                     *
# *******************************************************************************
my ($env, $db2sys, $menv, $os390_hostname, $os390_port, $stage,
    $copylib1, $copylib2, $copylib3, $copylib4,
    $syslib1, $syslib2, $syslib3,
#    $syslib4, $steplib1, $steplib2,			 # Optional
#    $hostpds_prefix, $hostpds_affix, $cbllib,	         # Optional 
    $loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) =
  bccpbmod::setEnvironmentVar();


print "Env: $env \n" if $debug;

# #############################################################################
# Translate the distributed username to a mainframe ID
#
my $pcid = $ENV{USERNAME};
my $mfid = bccpbmod::getUserID($pcid);                
bccpbmod::my_die('Unknown User.') if ($mfid eq "");   

# #############################################################################
# Set the local copylib and jcl/log directory for the current user
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You may hardcode PDS with a prefix or an affix.                             *
# *******************************************************************************
#my $name = "MFX." . $mfid;       # hardcode PDS with a prefix => MFX.USERID
my $name = $mfid . ".A";          # hardcode PDS with a affix  => USERID.A

# *******************************************************************************
# * Optional customization:                                                     *
# * The following code can be used to generate the PDS if the type attribute    *
# * ZCCENV is set with the $hostpds_prefix and $hostpds_affix.                  *
# *******************************************************************************
#
#  my $name = $mfid;
#  my $affix ="";
#  my $prefix ="";
#  if (($hostpds_prefix ne '') || ($hostpds_affix ne '')) {
#     if ($hostpds_prefix ne '') {
#        $prefix = $hostpds_prefix . ".";
#     }
#     if ($hostpds_affix ne '') {
#        $affix = "." . $hostpds_affix ;
#     }
#     $name = $prefix . $mfid . $affix ;
#   } 
################################################################################


print "PCID: $pcid, MFID: $mfid, NAME: $name\n" if $debug;

# #############################################################################
# determine the member name from the clearcase filename. The member name is
# just the filename without the front path and extension.
#
my $delimiter= '\\';
my $del_ind  = rindex( $filename, $delimiter);
my $mem_ext  = substr( $filename, ($del_ind == -1 ? 0: ($del_ind+1)), length($filename));
my $dot      = rindex( $mem_ext, '.' );
my $mem      = uc( substr( $mem_ext, 0, $dot ) );
my $ext      = substr( $mem_ext, $dot );

my $jcldir = $dir_file.$delimiter.$mfid.$delimiter.$mem;

printf( "processing file (%s)\n", $filename ) if $debug;
printf( "  mem (%s) ext (%s)\n", $mem, $ext ) if $debug;


# #############################################################################
# Retrieve file information from element attribute
#
my %CCattr = bccpbmod::getCCattrs("..\\..\\$filename");

while (($attr, $value) = each %CCattr) {
	print "Attribute: $attr\tValue: $value\n";
	if ($compile_1) {
		print STDERR "Attribute: $attr\tValue: $value\n";
	}
}

# *******************************************************************************
# * Required customization:                                                     *
# * The following code has to be modified to meet your site's requirement.      *
# *******************************************************************************
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# The main routine continues by calling each of the specific job routines.
# If any of the routines fail, this scripts exits with an error message.

my $DLL = 'F';
if ( $mem =~ /^\w{4}[S]\w{2,3}$/ ) {    # 5th Char = 'S' (Subroutine)
    $DLL = 'U';
}
print $DLL if $debug;

  bccpbmod::my_die('Link Error.')                      if ( LinkProg() > 4 );  

exit ;

##############################################################
# The following subroutines contain each of the JCL decks
# that execute specific mainframe tasks
#
# Each subroutine performs the following functions:
#    1) build JCL file name
#    2) write specific JCL to the file
#    3) execute the JCL
#    4) report status
#
##############################################################



# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< LinkProg >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub LinkProg {
    my $status = 0;


    bccpbmod::my_die('LinkBT Error.')                if ( linkBT() > 4 );
    bccpbmod::my_die('CopyBT Error.')                if ( copyBT() > 4 );
    bccpbmod::my_die('CopyDO Error.')                if ( copyDO() > 4 );


    return $status;
}



#
#
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< linkBT >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub linkBT {

    #   if ( $CCattr{'BT'} eq 'Y' ) {
    if ( $CCattr{'LNKDECK'} ne 'N' ) {              # add to test lnk job
    my $linkcntl = $CCattr{'LNKDECK'};
    $jcl = 'lnkbatch_' . $mem .'.jcl';
    $jcl = "$jcldir\\$jcl";
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    my $lopt_parm = 'SIZE=(512K,96K)';
    if ( $CCattr{'LOPT'} ne 'N' ) {
        $lopt_parm  = $CCattr{'LOPT'};
    }

    bccpbmod::writeFile( $jcl, $compile_1, "
//* *********************************************  LINK-EDIT THE PROGRAM
//* COBOL390 = Y
//LKED     EXEC PGM=IEWL,
//  PARM='$lopt_parm'
//OBJLIB   DD DSN=$name.XOBJLIB,DISP=OLD
//SYSLIN   DD DSN=$name.XLOADLIB($linkcntl),DISP=OLD
"    );

    if ( $env eq $bccpbmod::PROD_env) {			
# PROD
        bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=PS.UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::QA_env) {			
 # QA
         bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=PS.UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::DEV_env) {		
 # DEV
          bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=$syslib3,DISP=SHR
//*        DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=SYS1.SCEELKED,DISP=SHR
//*        DD DSN=PS.UTL.PGMLIB,DISP=SHR
//*        DD DSN=SYS3.DB2.TEST.DSNLOAD,DISP=SHR
"
         );
     }


     # check IMS/DL1
     if ( $CCattr{'DL1'} eq 'Y' ) {
         bccpbmod::writeFile( $jcl, $compile_1, "
//         DD DSN=SYS2.IMSVS.RESLIB.TEST,DISP=SHR
//RESLIB   DD DSN=SYS2.IMSVS.RESLIB.TEST,DISP=SHR
"
         );
     }



        if ( $CCattr{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=CICSTS13.SDFHEXCI
"
            );
        }

        if ( $CCattr{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=MQSERIES.SCSQLOAD
"
            );
        }


        if ( $CCattr{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNEXIT
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNLOAD
"
            );
        }

        if ( $CCattr{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=PROD.LOAD
"
            );
        }


        if ( $CCattr{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INDB2TSO)
"
            );
        }

        if ( $CCattr{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INMQBTCH)
"
            );
        }


        bccpbmod::writeFile(
            $jcl, $compile_1, "
//SYSLMOD  DD DSN=$loadlib,
//            DISP=SHR
//SYSUT1   DD UNIT=VIO,SPACE=(CYL,(2,3))
//SYSPRINT DD $sysout
//            $sysout1
"
        );

        my $cmd =
"$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b LINKBATCH -ft $jcl -n 4 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

    print "Step: LinkProj-LinkBT cmd: $cmd\n " if $debug;

    my $status = 0;
    $status = bccpbmod::rcc_system($compile_1, $cmd);
    printf( "linkBT Status was (%d)\n", $status ) if $debug;

   }  # add to submit link job 
    return $status;
}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< copyBT >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub copyBT {

    my $status = 0;

     if ( $CCattr{'BT'} eq 'Y' ) {     # test 
        $jcl = 'copybt_' . $mem .'.jcl';
        $jcl = "$jcldir\\$jcl";
        if(-f $jcl)
    		{
    			unlink($jcl);
   			}

        bccpbmod::writeFile(
            $jcl, $compile_1, "
//BTTRNLD  EXEC PGM=IKJEFT01
//SYSPRINT DD $sysout
//            $sysout1
//INDD     DD DSN=PSBH.PGMTEST,DISP=SHR
//OUTDD    DD DSN=$name.LDPREP,DISP=SHR
//SYSTSPRT DD $sysout
//            $sysout1
//SYSTSIN  DD *
 XMIT A.A DDNAME(INDD) MEMBER($mem) OUTDDNAME(OUTDD) NOLOG
"
        );

        my $cmd =
"$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b COPYBT -ft $jcl -n 4 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

        print "Step: LinkProj-CopyBT cmd: $cmd\n " if $debug;

        $status = bccpbmod::rcc_system($compile_1, $cmd);

        printf( "copyBT Status was (%d)\n", $status ) if $debug;

     }                                  # test  
    return $status;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< copyDO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub copyDO {

    my $status = 0;
     if ( $CCattr{'BT'} eq 'Y' ) {     # test 

    $jcl = 'copydo_' . $mem .'.jcl';
    $jcl = "$jcldir\\$jcl";
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    bccpbmod::writeFile(
        $jcl, $compile_1, "
//XFERLOD  EXEC PGM=IEFBR14
//TEMP     DD DSN=$name.LDPREP,RCCEXT=LOD,DISP=OLD
"
    );

    my $cmd =
"$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b COPYDO -ft $jcl -n 4 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

    print "Step: LinkProj-CopyDO cmd: $cmd\n " if $debug;

    $status = bccpbmod::rcc_system($compile_1, $cmd);

    printf( "copyDO Status was (%d)\n", $status ) if $debug;

    }                                   # test  
    return $status;
}


